<?php

namespace itguild\forms\app\controllers;

use itguild\forms\app\core\BaseController;
use itguild\forms\app\models\FormModel;
use itguild\forms\app\models\FormResModel;
use itguild\forms\app\models\InputValueModel;
use itguild\forms\debug\Debug;
use itguild\forms\exceptions\FormNotFoundException;
use itguild\forms\exceptions\FormResultNotFoundException;
use itguild\forms\Form;
use itguild\forms\JsonForm;
use itguild\forms\table\actionBtn\ToDeleteBtn;
use itguild\forms\table\actionBtn\ToEditBtn;
use itguild\forms\table\actionBtn\ToListBtn;
use itguild\forms\table\ListJsonTable;
use itguild\forms\table\Pagination;
use itguild\forms\table\ViewJsonTable;
use Twig\Error\LoaderError;
use Twig\Error\RuntimeError;
use Twig\Error\SyntaxError;
use Twig\TwigFunction;


class FormController extends BaseController
{
    /**
     * @throws SyntaxError
     * @throws FormNotFoundException
     * @throws RuntimeError
     * @throws LoaderError
     */
    public function indexAction($id): void
    {
        $form = FormModel::find($id);

        if (!$form) {
            throw new FormNotFoundException($id);
        }

        $formArr = $this->createFormArr($form);

        $formArr['data'][] = ['type' => "button", 'typeInput' => "submit", "value" => "Отправить", "name" => "", "class" => "btn btn-primary"];

        //Debug::dd($formArr);

        $this->view->addFunction(new TwigFunction("create_form", function () use ($formArr) {
            $form = new JsonForm(json_encode($formArr));
            $form->convertHTML();
            $form->render();
        }));

        echo $this->view->render('form/form.html.twig', ['title' => $form->title]);

    }

    public function saveAction($id): void
    {
        if (isset($_POST["form_res_id"])) {
            $formResId = $_POST["form_res_id"];
            unset($_POST["form_res_id"]);
            FormResModel::where("id", $formResId)->update(["data" => json_encode($_POST)]);
        } else {
            $formRes = FormResModel::Create(['form_id' => $id, 'data' => json_encode($_POST),]);
            $formResId = $formRes->id;
        }
        header('Location: ' . "/form-item/$formResId");
        exit;

    }


    public function createFormAction()
    {
        $form = Form::Create(['title' => "dsds", 'status' => 1, 'params' => "",]);
    }

    public function resultAction($id, $page = 1): void
    {

        $meta = [];
        $data = [];
        $tableArr = [];
        $columns = [];
        $columnsToMod = [];

        $form = FormModel::find($id);
        $formParams = json_decode($form->params, true);
        $perPage = $formParams['meta']['perpage'] ?? "10";
        $fillable = $formParams["meta"]["fillable"] ?? null;
        $actions = $formParams["meta"]["actions"] ?? null;
        $offset = ($page - 1) * $perPage;
        $countItems = FormResModel::where("status", 1)->count();

        $table = FormResModel::where("form_id", $id)->where("status", FormResModel::STATUS_ACTIVE)->limit(3)->offset($offset)->get();
        foreach ($table as $key => $item) {

            $data[] = array_merge(json_decode($item->data, true), ["id" => $item->id]);

        }

        if ($form->fields) {
            foreach ($form->fields as $field) {
                if ($field->input_type_id === 6) {
                    $fillable = $formParams["meta"]["fillable"] ?? null;
                    $columnsToMod[] = $field->name;

                }
                $columns[$field->name] = $field->label;


            }
        }

        $meta['title'] = "Form ID: " . $id;
        $meta['columns'] = $columns;
        $meta["perPage"] = $perPage;
        $meta["fillable"] = $fillable;
        $meta['params'] = ["class" => "table table-bordered", "border" => "1"];
        $meta['currentPage'] = $page;
        $meta['actions'] = $actions;
        $meta['baseUrl'] = BASE_URL;

        $tableArr["meta"] = $meta;
        $tableArr["data"] = $data;

        $tableRes = json_encode($tableArr);
        $this->view->addFunction(new TwigFunction("create_table", function () use ($tableRes, $columnsToMod, $form, $countItems) {

            $table = new ListJsonTable($tableRes);


            $table->setBeforePrintCell(function ($column, $data) use ($columnsToMod, $form) {
                if (in_array($column, $columnsToMod)) {
                    $res = InputValueModel::find($data);
                    return $res->value ?? "";
                }

                return $data;
            });
            $table->beforePrint(function () use($countItems){
                return "<h5 class='display-10 fw-bold'>Количество записей: " . $countItems . "</h5>";

            });

            $table->create();
            $table->render();


        }));

        $this->view->addFunction(new TwigFunction("create_pagination", function () use ($perPage, $page, $countItems, $form) {

            $pagination = new Pagination($countItems, $perPage, $page, BASE_URL . "/form-result/" . $form->id);
            $pagination->create();
            $pagination->render();

        }));
        echo $this->view->render('form/result.html.twig', ['title' => $tableArr["meta"]["title"]]);

    }

    public function deleteAction($id): void
    {
        $item = FormResModel::find($id);
        $item->update(["status" => FormResModel::STATUS_ARCHIVE]);

        header('Location: ' . "/form-result/$item->form_id");
        exit;
    }

    public function editAction($id): void
    {
        $res = FormResModel::find($id);
        if (!$res) {
            throw new  FormResultNotFoundException($id);
        }

        $form = FormModel::find($res->form_id);

        if (!$form) {
            throw new FormNotFoundException($id);
        }

        $formArr = $this->createFormArr($form, $res);
        $formArr["data"][] = ["type" => "hidden", "value" => $id, "name" => "form_res_id"];
        $formArr['data'][] = ['type' => "button", 'typeInput' => "submit", "value" => "Редактировать", "name" => "", "class" => "btn btn-primary"];

        //Debug::dd($formArr);

        $this->view->addFunction(new TwigFunction("create_form", function () use ($formArr) {
            $form = new JsonForm(json_encode($formArr));
            $form->convertHTML();
            $form->render();
        }));

        echo $this->view->render('form/form.html.twig', ['title' => $form->title]);

    }

    public function viewAction($id): void
    {
        $resultForm = FormResModel::find($id);
        $formID = $resultForm->form_id;
        $form = FormModel::find($formID);
        $meta = [];
        $columnsToMod = [];
        $columns = [];
        $resultData = $resultForm->data;


        if ($form->fields) {
            foreach ($form->fields as $field) {
                if ($field->input_type_id === 6) {
                    $columnsToMod[] = $field->name;

                }
                $columns[$field->name] = $field->label;


            }
        }


        $meta['title'] = "ITEM ID: " . $id;
        $meta['columns'] = $columns;
        $meta['params'] = ["class" => "table table-bordered", "border" => "1"];
        $meta["actions"] = ["view", "edit", "delete"];

        $tableArr["meta"] = $meta;
        $tableArr["data"] = $resultForm;

        $tableRes = json_encode($tableArr);


        $this->view->addFunction(new TwigFunction("create_get_action", function () use ($tableRes, $columnsToMod, $form, $resultForm) {

            $table = new ViewJsonTable($tableRes);
            $table->setBeforePrintCell(function ($column, $data) use ($columnsToMod, $form) {
                if (in_array($column, $columnsToMod)) {
                    $res = InputValueModel::find($data);
                    return $res->value ?? "";
                }

                return $data;
            });
            $table->beforeTable(function () use($form, $resultForm){
                $btn = (new ToListBtn(BASE_URL, $form->id))->fetch();
                $btn .= (new ToEditBtn(BASE_URL, $resultForm->id))->fetch();
                $btn .= (new ToDeleteBtn(BASE_URL, $resultForm->id))->fetch();
                return $btn;
            });

            $table->afterTable(function () use ($resultForm){

                $timeCreate = "<h5 class='display-10'>Создано: " . $resultForm->created_at . "</h5>";
                $timeUpdate = "<h5 class='display-10'>Обновлено: " . $resultForm->updated_at . "</h5>";;
                return $timeCreate . $timeUpdate;

            });

            $table->create();
            $table->render();

        }));

        echo $this->view->render('form/resultItem.html.twig', ['title' => $tableArr["meta"]["title"]]);
    }



    private function createFormArr(FormModel $form, FormResModel|false $formResult = false): array
    {
        // Инициализируем пустые массивы
        $meta = [];
        $formArr = [];
        $formFieldsArr = [];
        $resArr = [];
        // Получение формы из БД и обработка мета данных формы

        if ($formResult) {
            $resArr = json_decode($formResult->data, true);
        }

        if (isset($form->params)) {
            $allParams = json_decode($form->params, true);
            $meta = array_merge($meta, $allParams['params']);

        }
        $meta['action'] = "/form-save/" . $form->id;
        $meta['method'] = "POST";
        $formArr['meta'] = $meta;

        $formResults = FormResModel::where("form_id", $form->id)->where("id",);

        // Обработка полей формы
        if ($form->fields) {
            $fields = $form->fields;
            foreach ($fields as $field) {
                $options = [];
                $fieldArr = [];
                $params = [];
                if ($field->inputValue) {
                    foreach ($field->inputValue as $value) {
                        $options[$value['id']] = $value['value'];
                    }
                }

                if ($field->params) {
                    $fieldArr = array_merge($fieldArr, json_decode($field->params, true));
                }
                $fieldArr['name'] = $field->name ?? $fieldArr['name'];

                if ($field->input_type_id === 4) {
                    if ($resArr[$fieldArr['name']] === $fieldArr['value']) {
                        $fieldArr['checked'] = "true";
                    }
                }
                if ($field->input_type_id === 1) {
                    if ($resArr[$fieldArr['name']] === $fieldArr['value']) {
                        $fieldArr['checked'] = "true";
                    }
                }

                if ($formResult && $field->input_type_id !== 4) {
                    $fieldArr['value'] = $resArr[$fieldArr['name']] ?? null;
                }
                $fieldArr['type'] = $field->inputType['type'] ? $field->inputType['type'] : "textInput";
                $fieldArr['options'] = $options;
                $formArr['data'][] = $fieldArr;
            }
        }

        return $formArr;
    }


}