<?php

namespace itguild\forms\table;

use itguild\forms\debug\Debug;
use itguild\forms\table\ActionColumn\DeleteActionColumn;
use itguild\forms\table\ActionColumn\EditActionColumn;
use itguild\forms\table\ActionColumn\ViewActionColumn;
use itguild\forms\traits\CreateParams;
use itguild\forms\widgets\BaseWidget;

class ListJsonTable
{
    use CreateParams;

    private string $html = '';
    private string $json;

    private int $count = 0;
    private \Closure|false $beforePrintCell;
    private \Closure|false $beforePrint;
    private string $baseUrl;
    private array $data;

    private array $actionsArray = [];

    public function __construct(string $json)
    {
        $this->beforePrintCell = false;
        $this->json = $json;
        $this->data = json_decode($this->json, true);
        $this->baseUrl = $this->data['meta']['baseUrl'] ?? null;
        $this->setActions();
    }

    public function beginTable(): void
    {
        $paramsStr = $this->createParams($this->data['meta']['params']);
        $hook = $this->beforePrint;
        $this->html .= $hook();
        $this->html .= "<table $paramsStr>";
    }

    public function beforePrint(\Closure $closure): void
    {
        $this->beforePrint = $closure;
    }

    public function createThead(): void
    {
        if (!isset($this->data['meta']['columns'])) {
            return;
        }

        $this->html .= "<thead><tr>";
        $this->html .= "<th>" . "ID" . "</th>";
        foreach ($this->data['meta']['columns'] as $key => $column) {
            if ($this->is_fillable($key)) {
                $this->html .= "<th>" . $column . "</th>";
            }
        }
        $this->html .= "<th>Действия</th></th></tr></thead>";
    }

    public function createTbody(): void
    {
        if ($this->data['data']) {

            $this->count = $this->data["meta"]["perPage"] * ($this->data['meta']["currentPage"] - 1);
            foreach ($this->data['data'] as $col) {
                $this->html .= "<tr>";
                $this->count += 1;
                $this->html .= '<td><a href=' . $this->baseUrl . "/form-item/" . $col["id"] . '>' . $this->count . '</a></td>';
                foreach ($col as $key => $row) {
                    if ($this->issetColumn($key) and $this->is_fillable($key)) {
                        if ($this->beforePrintCell) {
                            $hook = $this->beforePrintCell;
                            $row = $hook($key, $row);
                        }

                        $this->html .= "<td>" . $row . "</td>";
                    }
                }

                $actions = $this->getActions($col["id"]);

                $this->html .= "<td>$actions</td></tr>";
            }
        }
    }

    private function setActions(): void
    {
        if (isset($this->data['meta']['actions'])) {
            foreach ($this->data['meta']['actions'] as $action) {
                switch ($action) {
                    case 'view':
                        $this->actionsArray[] = ViewActionColumn::class;
                        break;
                    case 'delete':
                        $this->actionsArray[] = DeleteActionColumn::class;
                        break;
                    case 'edit':
                        $this->actionsArray[] = EditActionColumn::class;
                        break;
                }
            }
        }
    }

    private function issetColumn($column)
    {
        if (isset($this->data['meta']['columns'])) {
            foreach ($this->data['meta']['columns'] as $key => $currentColumn) {
                if ($key === $column) {
                    return true;
                }
            }
        }

        return false;
    }

    private function is_fillable($column)
    {
        if (in_array($column, $this->data['meta']['fillable'])) {
            return true;
        }

        return false;
    }

    private function getActions(int $id): string
    {
        $actions = "";
        foreach ($this->actionsArray as $item) {
            $objItem = new $item($this->baseUrl, $id);
            $actions .= $objItem->fetch();
        }

        return $actions;
    }

    public function create()
    {
        $this->beginTable();
        $this->createThead();
        $this->createTbody();
        $this->endTable();
    }

    public function tableAction($json): void
    {

        $tableJson = json_decode($json, true);

        foreach ($tableJson as $key => $value) {
            echo "<tr>";
            echo "<td>" . $key . "</td>";
            echo "<td>" . $value . "</td>";
            echo "</tr>";
        }
    }

    public function endTable(): void
    {
        $this->html .= "</table>";
    }

    public function render(): void
    {
        echo $this->html;
    }

    public function setBeforePrintCell(\Closure $closure): void
    {
        $this->beforePrintCell = $closure;
    }
}